package cn.com.infosec.crypto.engines;

import cn.com.infosec.crypto.CipherParameters;
import cn.com.infosec.crypto.DataLengthException;
import cn.com.infosec.crypto.StreamCipher;
import cn.com.infosec.crypto.params.KeyParameter;

public class RC4Engine implements StreamCipher {
   private static final int STATE_LENGTH = 256;
   private byte[] engineState = null;
   private int x = 0;
   private int y = 0;
   private byte[] workingKey = null;

   private void setKey(byte[] keyBytes) {
      this.workingKey = keyBytes;
      this.x = 0;
      this.y = 0;
      if (this.engineState == null) {
         this.engineState = new byte[256];
      }

      int var2;
      for(var2 = 0; var2 < 256; ++var2) {
         this.engineState[var2] = (byte)var2;
      }

      var2 = 0;
      int var3 = 0;

      for(int var4 = 0; var4 < 256; ++var4) {
         int var10000 = var2;
         var2 = keyBytes[var2] & 255;
         byte[] var5;
         byte[] var10001 = var5 = this.engineState;
         var3 = var2 + var5[var4] + var3 & 255;
         byte var6 = var5[var4];
         var10001[var4] = var5[var3];
         var10001[var3] = var6;
         var2 = (var10000 + 1) % keyBytes.length;
      }

   }

   public void init(boolean forEncryption, CipherParameters params) {
      if (params instanceof KeyParameter) {
         RC4Engine var10000 = this;
         byte[] var3;
         this.workingKey = var3 = ((KeyParameter)params).getKey();
         var10000.setKey(var3);
      } else {
         throw new IllegalArgumentException("invalid parameter passed to RC4 init - " + params.getClass().getName());
      }
   }

   public String getAlgorithmName() {
      return "RC4";
   }

   public byte returnByte(byte in) {
      byte var10000 = (byte)in;
      this.x = in = this.x + 1 & 255;
      byte[] var2;
      byte[] var10001 = var2 = this.engineState;
      int var10003 = in;
      int var10006 = in;
      int var3;
      this.y = var3 = var2[in] + this.y & 255;
      byte in1 = var2[in];
      var2[var10006] = var2[var3];
      var2[var3] = in1;
      return (byte)(var10000 ^ var10001[var10001[var10003] + var2[var3] & 255]);
   }

   public void processBytes(byte[] in, int inOff, int len, byte[] out, int outOff) {
      if (inOff + len > in.length) {
         throw new DataLengthException("input buffer too short");
      } else if (outOff + len > out.length) {
         throw new DataLengthException("output buffer too short");
      } else {
         for(int var6 = 0; var6 < len; ++var6) {
            int var7;
            this.x = var7 = this.x + 1 & 255;
            byte[] var8;
            byte[] var10006 = var8 = this.engineState;
            int var9;
            this.y = var9 = var8[var7] + this.y & 255;
            int var10 = var8[var7];
            var10006[var7] = var8[var9];
            var10006[var9] = (byte)var10;
            var10 = var6 + outOff;
            out[var10] = (byte)(in[var6 + inOff] ^ var8[var8[var7] + var8[var9] & 255]);
         }

      }
   }

   public void reset() {
      this.setKey(this.workingKey);
   }
}
